function tau = supg_euler_1d(Upg,UPpg,v_mesh,gamma,hele,ndf,tau_TET,preco,dt)
%
% calculo de tau (stabilizacion numerica) para Euler 1D
%
%     tau = supg_euler_1d(Upg,UPpg,gamma,hele,ndf,tau_TET,preco,dt)
%

flag_debug = 0;

ne = length(Upg(:,1));

% calculo del upwind en cada punto de Gauss  
if tau_TET
    % segun Tezduyar
    cc = sqrt(gamma*UPpg(:,3)./UPpg(:,1));
    if preco
        xi = 1;
        delta = 1;
        omega = gamma-(gamma-1)*delta;
        Meps  = 1e-3;
        beta = cc.^2;
        M  = abs(UPpg(:,2))./cc;
        
        CFLc = cc*dt./hele;
        
        %% Mr = min([ones(size(M)) max([M Meps*ones(size(M))],[],2)],[],2);
        Mr = min([ones(size(M)) max([sqrt(M.^2+xi./CFLc.^2) Meps*ones(size(M))],[],2)],[],2);
        
        pseudo_c = sqrt(UPpg(:,2).^2.*(1+beta.*Mr.^2*omega./cc.^2).^2+ ...
			4*beta.*Mr.^2.*(1-UPpg(:,2).^2./cc.^2));

        lambda2    = 1/2*abs(UPpg(:,2).*(1+beta.*Mr.^2*omega./cc.^2)+pseudo_c);
        lambda3    = 1/2*abs(UPpg(:,2).*(1+beta.*Mr.^2*omega./cc.^2)-pseudo_c);
        lambda_max = max([abs(UPpg(:,2)) lambda2 lambda3],[],2);
        
        if flag_debug
            if 0
                lambda_theo = eigen_Av(UPpg,gamma,beta,Mr);
                err_max = max(abs(max(abs(lambda_theo'))'-lambda_max));
                if err_max > 1e-10, error(' *** SUPG_EULER_1D routine Error *** flag_debug [1] '); end
            else
                Av = Ajac_euler_1D(Upg,UPpg,gamma);
                dUdQ = dUdQv(UPpg,gamma);
                iGammav = iprecov(UPpg,gamma,dt,hele);
                maux1 = rowprod(Av,dUdQ,ndf,ndf,ndf);
                Av = rowprod(iGammav,maux1,ndf,ndf,ndf);
                
                for k=1:size(Av,1),
                    AAv = reshape(Av(k,:,:),3,3);
                    [Sv_theo,Lv_theo]=eig(AAv);
                    tau_v(k,:,:) = hele(k)/2*Sv_theo*inv(abs(Lv_theo))*inv(Sv_theo);                    
                end
            end
        end
        
    else
        lambda_max = abs(UPpg(:,2)-v_mesh)+cc;
    end
    tau  = hele/2 ./ lambda_max;
    tau_m  = zeros(ne,ndf,ndf);
    for idf=1:ndf
        tau_m(:,idf,idf) = tau;
    end
    tau = tau_m;
    
    if preco
        dUdQ = dUdQv(UPpg,gamma);
        iGammav = iprecov(UPpg,gamma,dt,hele);
        maux1 = rowprod(dUdQ,tau,ndf,ndf,ndf);
        maux2 = rowprod(maux1,iGammav,ndf,ndf,ndf);
        tau = maux2;
        
        if flag_debug
            maux1 = rowprod(dUdQ,tau_v,ndf,ndf,ndf);
            maux2 = rowprod(maux1,iGammav,ndf,ndf,ndf);            
            tau_c = maux2;                            
            tau = tau_c;
            %        keyboard;pause
        end
        
    end
    
else
  if preco
    error('No implementado! -> supg_euler_1d.m')
  else
    % segun una diagonalizacion y descomposicion en autovalores  
    [Q,Q1,D] = diagsys(Upg,gamma);
    %    Dm = zeros(ne,ndf,ndf);
    tau_m = zeros(ne,ndf,ndf);
    for k=1:ndf
      %        Dm(:,k,k)=D(:,k);
      tau_m(:,k,k) = (hele/2) ./ abs(D(:,k)-v_mesh);
    end    
    tau_m  = rowprod(Q,tau_m,ndf,ndf,ndf);
    tau_m  = rowprod(tau_m,Q1,ndf,ndf,ndf);   
    %    tau  = ((hele/2)*ones(1,ndf^2)).*tau_m;
    %    tau  = reshape(tau,ne,ndf,ndf);
    %    disp(' supg_euler_1d ');keyboard;pause
    tau = tau_m;
  end
end

%% disp('supg'),keyboard
