function Riemann_solver(uL,uR,t,xmin,xmax)

%
%   Riemann_solver(uL,uR,t,xmin,xmax)
%   u=[rho u p];
%

tol=1e-10;

gamma=1.4;

rho1=uR(1);
u1=uR(2);
p1=uR(3);
a1=sqrt(gamma*p1/rho1);

rho4=uL(1);
u4=uL(2);
p4=uL(3);
a4=sqrt(gamma*p4/rho4);

F=1+tol;
beta=0.1*p4/p1;

while abs(F) >= tol

  F=(p1/p4)^((gamma-1)/(2*gamma))-1/beta^((gamma-1)/(2*gamma))*(1+(gamma-1)/(2*a4)*(u4-u1- ...
     a1/gamma*(beta-1)/sqrt((gamma+1)/(2*gamma)*(beta-1)+1)));

  dF=(gamma-1)/(2*gamma*beta^((gamma-1)/(2*gamma)))*(1/beta*(1+(gamma-1)/(2*a4)*(u4-u1- ...
     a1/gamma*(beta-1)/sqrt((gamma+1)/(2*gamma)*(beta-1)+1)))+a1/a4*(1/sqrt((gamma+1)/(2*gamma)*(beta-1)+1)- ...
     (gamma+1)/(4*gamma)*(beta-1)/(sqrt((gamma+1)/(2*gamma)*(beta-1)+1))^3));

  beta=beta-F/dF;

end

% Estado 2

p2=p1*beta;

u2=u4+2*a4/(gamma-1)*(1-(p2/p4)^((gamma-1)/(2*gamma)));

a2=a1*sqrt(p2/p1*((gamma+1)/(gamma-1)+p2/p1)/(1+(gamma+1)/(gamma-1)*p2/p1));

% Velocidad del choque

S=u1+a1*sqrt((gamma+1)/(2*gamma)*(p2/p1-1)+1);

% Estado 2

p3=p2;

u3=u2;

a3=(gamma-1)/2*(u4-u3)+a4;

% Estado dentro del abanico de expansin

N_ae=100;

x_ae=(u4-a4)*t:((u3-a3-u4+a4)*t/N_ae):(u3-a3)*t;

u_ae=2/(gamma+1)*(x_ae/t+(gamma-1)/2*u4+a4);

a_ae=u_ae-x_ae/t;

p_ae=p4*(a_ae/a4).^(2*gamma/(gamma-1));

% Solucin

x=[xmin x_ae u3*t u3*t S*t S*t xmax];

p=[p4 p_ae p3 p2 p2 p1 p1];

u=[u4 u_ae u3 u2 u2 u1 u1];

a=[a4 a_ae a3 a2 a2 a1 a1];

rho=gamma*p./a.^2;

s=1/(gamma-1)*(log(p/p1)-gamma*log(rho/rho1));

M=u./a;

figure(1)
plot(x,p)
xlabel('x'),ylabel('Presion')

figure(2)
plot(x,u)
xlabel('x'),ylabel('Velocidad')

figure(3)
plot(x,a)
xlabel('x'),ylabel('Velocidad del sonido')

figure(4)
plot(x,rho)
xlabel('x'),ylabel('Densidad')

figure(5)
plot(x,s)
xlabel('x'),ylabel('Entropia / R')

figure(6)
plot(x,M)
xlabel('x'),ylabel('Numero de Mach')
