%
%   Example 6 : 
%           Advection with source Q(x) = 1 
%           phi(x=0) = 0 and phi(x=Lx) = 0
%

nen_ele  = 2 ; Ne=20; Nx=Ne*(nen_ele-1)+1;

Lx       = 1;
psi      = (0:Nx-1)'/(Nx-1);
x        = psi*Lx;
%x        = psi.^2*Lx;
%x        = (1+sin(-pi/2+psi*pi))/2;    % non uniform grid (refined at both ends)
hierarchical_flag = 0;

if exist('nen_ele','var')==0, nen_ele=2; end

%-------- coordinates (XNOD)
xnod= [x];

%-------- connectivities (ICONE)
if nen_ele==2;
    type_geo_elem   = 'cartesian1d';
    type_x_interp   = 'linear';
    type_sol_interp = 'linear'; 
    ico = [ (1:Nx-1)' (2:Nx)' ];     
elseif nen_ele==3
    type_geo_elem   = 'cartesian1d';
    type_x_interp   = 'linear';
    type_sol_interp = 'quadratic';    
    if hierarchical_flag == 1;
        ico  = [(1:2:Nx-2)' (3:2:Nx)' (2:2:Nx)' ]
    else
        ico  = [(1:2:Nx-2)' (2:2:Nx)' (3:2:Nx)'];          
    end   
elseif nen_ele==4
    type_x_interp   = 'linear';
    type_sol_interp = 'cubic';        
    if hierarchical_flag == 1;
        ico  = [(1:3:Nx-3)' (4:3:Nx)' (2:3:Nx-2)' (3:3:Nx-1)' ]; 
    else
        ico  = [(1:3:Nx-3)' (2:3:Nx-2)' (3:3:Nx-1)' (4:3:Nx)'];  
    end
end
icone = ico;    

%-------- fixations (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
ndf = 1;
nodes_lef  = 1;
nodes_rig  = Nx;
xmin = min(xnod);
xmax = max(xnod);
tol = 1e-5;
nodes_lef = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig = find(abs(xnod(:,1)-xmax(1))<tol);

fixa= zeros(numnp,ndf);
fixa(nodes_lef,1)=ones(length(nodes_lef),1);
%fixa(nodes_rig,1)=ones(length(nodes_rig),1);

%-------- values of fixations (Dirichlet and Neumann)
forc= zeros(numnp,ndf);
forc(nodes_lef,1)=0*ones(length(nodes_lef),1);
%forc(nodes_rig,1)=1*ones(length(nodes_rig),1);

%-------- Source term (by element) -----------------------------
xc_ele = 0.5*(xnod(icone(:,end),1)+xnod(icone(:,1),1));
if 1
    %source = -6*xc_ele;
    %source = (xc_ele<=Lx/2);
    %source = 1*ones(numel,1);    
    source = sin(xc_ele);
else
    source = zeros(numel,1);
    ffun_source = 'ffun_Ej_2_source';
end

%--------- Reactive term ( by element) -----------------------------
reac   = 0*ones(numel,1);

%--------- Velocity field -----------------------------
veloc  =  1*ones(numnp,ndm);

%-------- Material properties 
Pe = 1;
he = min(diff(xnod));
kappa    = max(abs(veloc))*he/2;
kappa    = 0*1e-5;
npg      =  2 ;
npg      =  3 ;
upwind   =  1 ;
upwind_type = 'Petrov_Galerkin';
%upwind_type = 'Bubble_function';
lumped   =  0 ;
nolineal =  0 ;
nolineal_function = 'ff_Ej_00';

ijob = 'set';
handle_prop_elem;

%-------- distributed fluxes
strg    = [];

%-------- Mixed boundary condition (Robin type) (TECO)
%                -k du/dn + h_film(u-u_0) = 0
% format for teco   
%               teco = [eles face val1 val2 ];
% with 
%
% eles    = element @ boundary;
% face    = face of the element @ boundary 
% val1    = h_film
% val2    = u_0
teco     = [];

%-------- periodic condition
lin_con = [];

%-------- Initial solution 
if 1
    phi0 =  0;
else
    ga = veloc ./ kappa;
    phi0 = 1 ./ veloc .* ((1-exp(ga.*xnod)) ./ (1-exp(ga)));
end

bini = zeros(numnp,ndf);
for k=1:ndf,
 bini(:,k)   = forc(:,k);
 free        = find(fixa(:,k)==0);
 bini(free,k)= phi0(1,k)*ones(size(free));
end

