function [Kg,Rg] = advdif_1d(data,states,jobinfo)
%
%    [Kg,Rg] = advdif_1d(data,states,jobinfo)
%
%        #.- advection, diffusion, reaction PDE in 1D with source and
%        temporal term
%
%        jobinfo = [1,1,0] ---> tangent matrix and residue
%        jobinfo = [0,1,0] ---> only residue
%        jobinfo = [0,1,1] ---> residue and mass matrix (for explicit schemes)
%
%------------------------------------------------------------------

% unpack data
xnod  = data.mesh.xnod;
icone = data.mesh.icone;
phi   = states.phi;
phiold   = states.phiold;
forc  = data.bc.forc;
teco  = data.bc.teco;
lin_con = data.bc.lin_con;
prop = data.props;

[numel,nen]   = size(icone);
[numnp,ndf]   = size(phi);
[numnp,ndm]   = size(xnod);

ijob = 'get';
handle_prop_elem;

dt       = data.props.dt;
tol      = data.props.limit;

if ndm==1,
    pgauss1d;
elseif ndm==2, 
    pgauss2d; 
elseif ndm==3,
    pgauss3d; 
end

T        = phi(:,1);
Told     = phiold(:,1);

veloc    = data.PDE_param.veloc;
reac     = data.PDE_param.reac;
source   = data.PDE_param.source;

%Rg       = forc;
Kele     = zeros(numel,nen,ndf,nen,ndf);
Rele     = zeros(numel,nen,ndf);

advdif_1d_ele;

% Mixed boundary condition
%------------------------------------------------------------------
if isempty(teco)==0,bc_teco; end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
[Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);

% Linear restrictions
%------------------------------------------------------------------
if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end
