%------------------------------------------------------------------
%       " Driver of F E M C O D E " 
%------------------------------------------------------------------

if(exist('initia')==0),
    disp('Enter variable INITIA ')
    disp(' initia == 0 ---> if is a continuation of an older run  ')
    disp(' initia ~= 0 ---> if started from the beginning  ')
    initia = input(' Initia = ')
end

%------ Input data -------------------------------------------

if initia,
    read_data;
    hc_res=[];       % storage for the residual convergence history 
    hc_du=[];        % storage for the solution increment convergence history 
    bfh_nc = [];     % Storage the steps non converged
    bfh_u=[];        % append states to plot the evolution
    posdat;
    ttime = 0;       % initial clock
    [rowe,cole] = gen_matrix_ij(icone,ndf);
    
end

%------ end of input data --------------------------------------

limit = data.props.limit;
ntime = data.props.ntime;
dt    = data.props.dt;
phi   = states.phi;
states.phiold = phi;    

data.system.row = rowe;
data.system.col = cole;

% Temporal loop 
%------------------------------------------------------------------

for itime=1:ntime,
    ttime = ttime+dt;
    
    phiold = phi;
    
    % Update the time dependent variables, parameters and or boundary conditions 
    % This functionality is in the "bc_time.m" routine (in the current
    % directory)
    if exist('bc_time_flag')==1,
        if (bc_time_flag==1)&(exist('bc_time')~=2), 
            disp('Debe tener en el directorio de trabajo un archivo de nombre bc_time.m ');
            error(' bc_time_flag==1 y bc_time file no existe '); 
        end
        if bc_time_flag==1, bc_time; end
    end
    
    nu    = 0;
    iterflag = 1;
    [numnp,ndf] = size(phi);
    
    while iterflag
        
        nu= nu+1;
        phi_nu = phi;
        
        jobinfo = [1,1,0];  % default for implicit matrix and vector assembling
        if type_solver ==2, jobinfo=[0,1,1]; end  % for explicit resolution (mass matrix and residue)

        % assembling the residual (Rg) and the tangent matrix (Kg)
        % [Kg,Rg] = advdif_1d(data,states,jobinfo);
        [Kg,Rg] = assemble(data,states,jobinfo,'advdif_1d');
        
        % Solving the linear system
        %------------------------------------------------------------------
        [phi,Rg]=solver(phi,Rg,Kg,fixa,data.props,1);
        
        % Convergence analysis
        %------------------------------------------------------------------
        iterflag = (norm(Rg)>limit) & (nu< nu_max);
        dphi = phi-phi_nu;
        
        disp(['time ' num2str(itime) ... 
                ' Iter ' num2str(nu)  ' || R || -----> ' num2str(norm(Rg)) ... 
                ' || Delta phi || -----> ' num2str(norm(dphi))  ])
        
        hc_res  = [hc_res ; norm(Rg)];
        hc_du   = [hc_du ; norm(dphi)];
        
        states.phi = phi;
        
    end  
    
    % Plot the results (plot_fem routine should be customized for each
    % application)
    %------------------------------------------------------------------    
    plot_fem
    
    if norm(Rg)>limit & type_solver ~=2 ,
        disp([' NO CONVERGENCE at time step ' num2str(itime) ' . The simulation will follow anyway' ])
        %  error(' NO CONVERGENCE ');
        bfh_nc = [bfh_nc ; itime];
    end
    
    if exist('nsave')==0; nsave=ntime;end
    
    if exist('filesave')==0; filesave='tempo'; end
    
    if rem(itime,nsave)==0, 
        eval(['save ' filesave ';']);
    end
    
    if rem(itime,nstep)==0,
        bfh_u = [bfh_u , phi];
    end
    
    %disp('DRVFEM');keyboard;pause    
    states.phi = phi;    
    states.phiold = phi;    
    
end 

