%
%   Example 3 : 2D heat equation
%               Fixed temperature at left and right walls
%               isolated at bottom and top boundaries
%               no source term 
%               linear reactive term  ==> phi_{xx} - phi = 0
%               no convection 
%               steady solution
%

Nx       = 41;Lx=1;  
Ny       = 5;Ly=0.1;  
nen_ele  = 2;

psi      = (0:Nx-1)'/(Nx-1);
x        = psi.^1*Lx;
eta      = (0:Ny-1)'/(Ny-1)*1; 
y0       = -1;
y        = eta.^1*Ly+y0;

%-------- coordenadas (XNOD)
xnod     =[];
for k=1:Ny,
    xnod=[xnod;[x ones(Nx,1)*y(k)]];
end

if nen_ele==2;  % linear approximation
    type_geo_elem   = 'cartesian2d';
%    type_geo_elem   = 'triangle';
    type_x_interp   = 'linear';
    type_sol_interp = 'linear'; 
    npg             =  2;
    
    %-------- conectividades (ICONE)
    icone    =[];
    ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
    ico2= [ (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)' (1:Nx-1)'];
    for k=1:Ny-1,
        icone=[icone; (k-1)*Nx+ico ];
    end
    
    if strcmp(type_geo_elem,'triangle')==1,
        % convert quads into triangles
        quad_to_tri;
        npg = 3;
    end
    
end
%-------- fijaciones (FIXA)
[numnp,ndm]=size(xnod);
[numel,nen]=size(icone);
ndf = 1;  % nunmber of degrees of freddom at each node

% identification of boundary faces for a simple cartesian geometry
xmin = min(xnod);
xmax = max(xnod);
tol  = 1e-5;

nodes_lef = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig = find(abs(xnod(:,1)-xmax(1))<tol);

nodes_bot = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top = find(abs(xnod(:,2)-xmax(2))<tol);

% boundary conditions
fixa= zeros(numnp,ndf);
forc = zeros(numnp,ndf);

% Dirichlet at the whole boundary
nodes_wall = rmdblist([nodes_lef;nodes_rig]);
fixa(nodes_wall,1) = 1;
forc(nodes_lef,1) = 0;
forc(nodes_rig,1) = 1;

%-------- Element-wise source
if 0
    xc=xc_ele(xnod,icone);
    xs_0 = xmin(1)+(xmax(1)-xmin(1))/3;
    xs_1 = xmin(1)+2*(xmax(1)-xmin(1))/3;
    ys_0 = xmin(2)+(xmax(2)-xmin(2))/3;
    ys_1 = xmin(2)+2*(xmax(2)-xmin(2))/3;    
    mask = (xc(:,1)>xs_0)&(xc(:,1)<xs_1)&(xc(:,2)>ys_0)&(xc(:,2)<ys_1);
    source = -0.1*mask;
else
    source = 0*ones(numel,1);
end

%--------- Termino reactivo -----------------------------
reac = ones(numel,1);

%--------- Campo de Velocidad -----------------------------
veloc  =  0*ones(numnp,1)*[cos(pi/3) sin(pi/3)];
%-------- Propiedades del material

rho     =    1;
Cp      =    1;
kappa   =    1;
upwind  =    0;
shocap  =    0;
axisim  =    0;
tau_fac =    0;
% [1] SUPG
% [2] (SU+C)PG
type_upw =    1;   
lumped   =   0 ;

ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
strg    = [];

%-------- Condiciones de contorno mixtas (TECO)
if 0
eles    = (2*(Nx-1):Nx-1:(Nx-1)*(Ny-1))';
[numel,nen]=size(icone);
face    = 2*ones(size(eles));
val1    = 800*ones(size(eles));
val2    = 40*ones(size(eles));
teco    = [eles face val1/rho/Cp val2 ];
else
teco     = [];
end
%-------- Condiciones periodicas
if 0
uno  = ones(length(nodes_bot),1);
lin_con = [uno nodes_bot uno -uno nodes_top uno];
else
lin_con = [];
end

%-------- Solucion inicial (PHI)
phi0 =  0;
bini = zeros(numnp,ndf);
for k=1:ndf,
    bini(:,k)   = forc(:,k);
    free        = find(fixa(:,k)==0);
    bini(free,k)= phi0(1,k)*ones(size(free));
end

adre = 0;
filename = 'adre_advdif_01'

% Type_solver ------>  [ 0 ]  standard - direct method with relaxation (relax)
%                      [ 1 ]  line-search
%                      [ 2 ]  explicit
%                      [ 3 ]  GMRES
%                      [ 4 ]  QMR
type_solver = 0;

limit       = 1e-06;
dt          = 0.0001 ;
relax       = 1;     % relaxation parameter
nu_max      = 1;	 % maximum number of iterations each time step 
alfa_min    = 0.01;  % parameter for linesearch
alpha       = 1;     % trapezoidal time integration (1/2 Cranck-Nicholson)
ntime       = 1;    % number of time steps 
nstep       = 1;     % append the solution each nstep time steps in bfh_u array
scaling     = 1;     % scaling equations at solver for better conditioning of the system
nplot       = 1;     % plot each "nplot" time steps
nsave       = 5;     % save in memory the whole memory each nsave time steps
