function [Kg,Rg] = advdif_2d(data,states,jobinfo)
%
%    [Kg,Rg] = advdif_1d(data,states,jobinfo)
%
%        #.- advection, diffusion, reaction PDE in 2D with source and
%        temporal term for triangular and quadrangular elements in plane
%        and axisymmetric geometries
%
%        jobinfo = [1,1,0] ---> tangent matrix and residue
%        jobinfo = [0,1,0] ---> only residue
%        jobinfo = [0,1,1] ---> residue and mass matrix (for explicit schemes)
%
%------------------------------------------------------------------

% unpack data
xnod     = data.mesh.xnod;
icone    = data.mesh.icone;
phi      = states.phi;
phiold   = states.phiold;
forc     = data.bc.forc;
teco     = data.bc.teco;
lin_con  = data.bc.lin_con;
prop     = data.props;
ther_contact = data.bc.ther_contact;

[numel,nen]   = size(icone);
[numnp,ndf]   = size(phi);
[numnp,ndm]   = size(xnod);

ijob = 'get';
handle_prop_elem;

dt       = data.props.dt;
tol      = data.props.limit;

if ndm==1,
    pgauss1d;
elseif ndm==2, 
    pgauss2d; 
elseif ndm==3,
    pgauss3d; 
end

if ndf~=1, error(' NDF debe ser 1');end

alpha    = prop.alpha;
dt       = prop.dt;
tol      = prop.limit;

%dt       = alpha*dt;
T        = alpha*phi(:,1)+(1-alpha)*phiold;

Told = phiold(:,1);

veloc    = data.PDE_param.veloc;
reac     = data.PDE_param.reac;
source   = data.PDE_param.source;

Rg   = forc;
Kele = zeros(numel,nen,ndf,nen,ndf);
Rele = zeros(numel,nen,ndf);
mass_lumped = zeros(numnp,1);

if ~isempty(ther_contact), 
    numel_th = length(ther_contact(:,1));
    Kele_th =zeros(numel_th,4,ndf,4,ndf);
    Rele_th =zeros(numel_th,4,ndf);     
end  

eles = (1:numel);
advdif_2d_ele;

% ------ Terminos de contorno
eles = (1:numel)';  
ceros = zeros(length(eles),nen);
Te = ceros;   
vele = zeros(length(eles),nen,ndm);  
for inod=1:nen,
    nodes = icone(eles,inod);
    Te(:,inod) = T(nodes);
    vele(:,inod,1:ndm) = veloc(nodes,1:ndm);       
end

% Convective heat flux at boundary
%------------------------------------------------------------------
if ~isempty(teco), 
    [Kele,Rele] = bc_teco_advdif_2d(xnod,icone,Te,teco,Kele,Rele,ijob,axisim);
end

if weak_form,
    [Rele,Kele] = bcconv_advdif_2d(xnod,icone,vele,Te,ifaceb,Rele,Kele,axisim);
end

% Thermal contact 
%------------------------------------------------------------------
if ~isempty(ther_contact), 
    [Kele_th,Rele_th] = bc_thermal_contact(xnod,icone,T,ther_contact,Kele_th,Rele_th,ijob,axisim);
end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
if ~isempty(ther_contact), 
    icone_th = ther_contact(:,1:4);
    [Rg,Kg] = gather([Rele;Rele_th],[Kele;Kele_th],[icone;icone_th],nen,ndf);
else
    [Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);
end

if 0,
    Rg = Rg - diag(mass_lumped)*phi;
    Kg = Kg + diag(mass_lumped);
end

if 0,
    Rg = Rg+forc;
end

% Linear restrictions
%------------------------------------------------------------------
if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end

% disp('fem001');keyboard;pause