% ====================================================================
%
%                       Ejemplo lateral maquina cosechadora aceituna
%
% ====================================================================

read_from_file = 0;
ndf      = 6;
tol      = 1e-6;
npg      = 2 ;

axisim   = 0 ;
plane_stress = 1;
beam_theory = 1;

% PHYSICAL DATA
rho      = 7800 ;
gravity  = 0*9.81;
young    = 2.15e11 ; % en Pa=N/m^2 (acero)
poisson  = 0.25 ;  
g_dir    = 3;

% DATOS GEOMETRICOS DE LAS VIGAS
width  = 80/1000;
height = 80/1000;
thickness = 3.2/1000;

% INITIAL VALUES
uini    = 1e-3*[0,0,0,0,0,0];

% ====================================================================
xnod = [ ... 
        1 0 0 0
    2 1150 0 -1430
    3 2850 0 -1430
    4 4230 0 0
    5 4230 0 1570
    6 4230 0 2570
    7 2000 0 2570
    8 0 0 2570
    9 0 0 1570
    10 2000 0 1570
    11 2000 0 0
    12 0 1000 0
    13 1150 1000 -1430
    14 2850 1000 -1430
    15 4230 1000 0
    16 4230 1000 1570
    17 4230 1000 2570
    18 2000 1000 2570
    19 0 1000 2570
    20 0 1000 1570
    21 2000 1000 1570
    22 2000 1000 0
    23 2482 0 -811
    24 2482 1000 -811
    25 670 0 2570
    26 670 0 1570
    27 2000 0 2070
    28 4230 500 2070 ];

xnod=xnod(:,2:4)/1000;

icone = [...
        1   1   2   
    2   2   3   
    3   3   4   
    4   4   6   
    5   6   8   
    6   8   1   
    7   1   4   
    8   9   5   
    9   7   10  
    10  10   11 
    11  11   3  
    12  4   23  
    13  23   2  
    14  12   13 
    15   13   14
    16   14   15
    17   15   17
    18   17   19
    19   19   12
    20   12   15
    21   20   16
    22   18   21
    23   21   22
    24   22   14
    25   15   24
    26   24   13
    27   1   12 
    28   4   15 
    29   5   16
    30   9   20
    31   8   19
    32   7   18
    33   6   17
    34   10   21
    35   11   22
    36   23   24
    37   1   22
    38   22   4
    39   9   12
    40   5   15
    41   11   21
    42   9   19
    43   5   17
    44   7   21
    45   19   25
    46   25   18
    47   18   6
    48   6   28
    49   28   16
    50   21   5
    51   26   21
    52   20   26
    53   5   27
    54   6   27
    55   20   22
    56   9   11
    57   22   23 ];
icone = icone(:,2:3);

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

ncol_icone_prop = 6;
icone_prop = zeros(numel,ncol_icone_prop);

% MOMENTOS DE INERCIA EN EJES LOCALES
if 0
    D_ext = width;
    D_int = width-thickness;
    I_xx = pi/64*(D_ext^4-D_int^4)*ones(numel,1);
    I_yy =  I_xx;
    I_zz =  I_xx+I_yy;
    I_x  =  ones(numel,1);
    I_y  =  ones(numel,1);
    Area = pi/4*(D_ext^2-D_int^2)*ones(numel,1);
else
    width_p  = width-2*thickness;
    height_p = height-2*thickness;
    
    I_xx =  width*height^3/12;
    I_xx =  I_xx - width_p*height_p^3/12;
    I_xx =  I_xx*ones(numel,1);
    
    I_yy =  width^3*height/12;
    I_yy =  I_yy - width_p^3*height_p/12;
    I_yy =  I_yy*ones(numel,1);
    
    I_zz =  I_xx+I_yy;
    
    I_x  =  0*ones(numel,1);
    I_y  =  0*ones(numel,1);
    
    Area =  width*height;
    Area =  Area - width_p*height_p;
    Area =  Area*ones(numel,1);
    
    
end

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate '.mat']);
    U = phi;
else
    U       =  ones(numnp,1)*uini;   
end

xmin = min(xnod);
xmax = max(xnod);
nodes_lef = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig = find(abs(xnod(:,1)-xmax(1))<tol);

%-------- fijaciones (FIXA)
fixa= zeros(numnp,ndf);
fixa([17,18,19],:) = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc   = zeros(numnp,ndf);
forc([2,3,13,14],2) = 2500;
%node_half = Nx/2+1;
%forc(node_half,2) = -1;

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
bou_strg = [];
if 0,
    [ifaceb,nodes_bou]=pfbou2d(xnod,icone,1);
    
    eles_rig = get_eles_by_nodes(nodes_rig,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_rig,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [ifaceb(eles_rig,1) , ones(length(eles_rig),1)*2 , -tx*[1,0]]; 
    
    eles_lef = get_eles_by_nodes(nodes_lef,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_lef,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [bou_strg;[ifaceb(eles_lef,1) , ones(length(eles_lef),1)*4 , -tx*[1,0]]]; 
end

%-------- Condiciones de contorno mixtas (TECO)
%teco     = [];

% external volumetric force
source = zeros(numel,3);
%source(:,2) = 1;

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial 
%-------------------
bini   = U;

for k=1:ndf,
    nofree         = find(fixa(:,k)~=0);
    bini(nofree,k) = forc(nofree,k);
end   

bc_time_flag = 0;
bc_time_t_end = 1;
