% ====================================================================
%
%                       Ejemplo Oate pag 114
%     viga en voladizo con 
%   (a) carga repartida a lo largo de su longitud
%   (b) carga puntual en el extremo libre
%
% ====================================================================

L        = 1;                     % Depth of the fluid

Nx       = 32;
read_from_file = 0;
ndf      = 6;
tol      = 1e-6;
npg      = 2 ;

axisim   = 0 ;
plane_stress = 1;
beam_theory = 1;

% PHYSICAL DATA
rho      = 7900;

gravity  = -1*9.81;
young    = 2.E+11 ;
poisson  = 0.29 ;  
g_dir    = 2;

% DATOS GEOMETRICOS DE LA VIGA
width  = 80/1000;
height = 120/1000;
thickness = 5/1000;

% INITIAL VALUES
uini    = 1e-3*[0,0,0,0,0,0];

% PUNCTUAL FORCE
force_p = -10000;

% ====================================================================
dx = L/Nx;
xnod = [0,0,0];
icone = [];
for k=1:Nx
    xnod   = [xnod;[k*dx,0,0]];
    icone  = [icone;[k,k+1]];
end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

ncol_icone_prop = 6;
icone_prop = zeros(numel,ncol_icone_prop);

width_p = width-2*thickness;
height_p = height-2*thickness;

I_xx =  width*height^3/12;
I_xx =  I_xx - width_p*height_p^3/12;
I_xx =  I_xx*ones(numel,1);

I_yy =  width^3*height/12;
I_yy =  I_yy - width_p^3*height_p/12;
I_yy =  I_yy*ones(numel,1);

I_zz =  I_xx+I_yy;

I_x  =  0*ones(numel,1);
I_y  =  0*ones(numel,1);

Area =  width*height;
Area =  Area - width_p*height_p;
Area =  Area*ones(numel,1);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate '.mat']);
    U = phi;
else
    U       =  ones(numnp,1)*uini;   
end

xmin = min(xnod);
xmax = max(xnod);
nodes_lef = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig = find(abs(xnod(:,1)-xmax(1))<tol);

%-------- fijaciones (FIXA)
fixa= zeros(numnp,ndf);
fixa(:,[3,4,5]) = 1;
fixa(nodes_lef,1:6) = 1;
%fixa(nodes_rig,1:2) = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc   = zeros(numnp,ndf);
forc(numnp,2) = force_p;
%node_half = Nx/2+1;
%forc(node_half,2) = -1;


%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
bou_strg = [];
if 0,
    [ifaceb,nodes_bou]=pfbou2d(xnod,icone,1);
    
    eles_rig = get_eles_by_nodes(nodes_rig,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_rig,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [ifaceb(eles_rig,1) , ones(length(eles_rig),1)*2 , -tx*[1,0]]; 
    
    eles_lef = get_eles_by_nodes(nodes_lef,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_lef,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [bou_strg;[ifaceb(eles_lef,1) , ones(length(eles_lef),1)*4 , -tx*[1,0]]]; 
end

%-------- Condiciones de contorno mixtas (TECO)
%teco     = [];

% external volumetric force
source = zeros(numel,3);
%source(:,2) = 1;

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial 
%-------------------
bini   = U;

for k=1:ndf,
    nofree         = find(fixa(:,k)~=0);
    bini(nofree,k) = forc(nofree,k);
end   

bc_time_flag = 0;
bc_time_t_end = 1;
