% ====================================================================
%
%                       Poiseuille flow
%
% ====================================================================

L        = 1;                     % Depth of the fluid

read_from_file = 0;
ndf      = 6;
tol      = 1e-6;
npg      = 2 ;

axisim   = 0 ;
plane_stress = 1;
beam_theory = 1;

% PHYSICAL DATA
rho      = 7800 ;
gravity  = -1*9.81;
young    = 1e6 ;
poisson  = 0.25 ;  
g_dir    = 2;

% INITIAL VALUES
uini    = 1e-3*[0,0,0,0,0,0];

% ====================================================================
xnod     = [0,L,0; L,L,0 ; 0,0,0];
icone    = [1,2;2,3];

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

ncol_icone_prop = 6;
icone_prop = zeros(numel,ncol_icone_prop);

I_xx =  ones(numel,1);
I_yy =  ones(numel,1);
I_zz =  ones(numel,1);
I_x  =  ones(numel,1);
I_y  =  ones(numel,1);
Dia  = 3.2/1000;
Area =  ones(numel,1)*(pi/4*Dia^2);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate '.mat']);
    U = phi;
else
    U       =  ones(numnp,1)*uini;   
end

%-------- fijaciones (FIXA)
fixa= zeros(numnp,ndf);
fixa(:,3:6) = 1;
fixa(1,1:2) = 1;
fixa(3,1:2) = 1;

%-------- Cargas/Desplazamientos nodales(FORC)
forc   = zeros(numnp,ndf);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
bou_strg = [];
if 0,
    [ifaceb,nodes_bou]=pfbou2d(xnod,icone,1);
    
    eles_rig = get_eles_by_nodes(nodes_rig,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_rig,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [ifaceb(eles_rig,1) , ones(length(eles_rig),1)*2 , -tx*[1,0]]; 
    
    eles_lef = get_eles_by_nodes(nodes_lef,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_lef,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [bou_strg;[ifaceb(eles_lef,1) , ones(length(eles_lef),1)*4 , -tx*[1,0]]]; 
end

%-------- Condiciones de contorno mixtas (TECO)
%teco     = [];

% external volumetric force
source = zeros(numel,3);

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial 
%-------------------
bini   = U;

for k=1:ndf,
    nofree         = find(fixa(:,k)~=0);
    bini(nofree,k) = forc(nofree,k);
end   

bc_time_flag = 0;
bc_time_t_end = 1;
