function [Kg,Rg] = assemble(data,states,jobinfo)
%
%     [Kg,Rg] = assemble(data,states,jobinfo)
%
%  ASSEMBLING element contributions to matrix and residual
%
%        jobinfo = [1,1,0] ---> tangent matrix and residue
%        jobinfo = [0,1,0] ---> only residue
%        jobinfo = [0,1,1] ---> residue and mass matrix (for explicit schemes)
%
%------------------------------------------------------------------

% unpack data
xnod     = data.mesh.xnod;
icone    = data.mesh.icone;
phi      = states.phi;
phiold   = states.phiold;
forc     = data.bc.forc;
teco     = data.bc.teco;
lin_con  = data.bc.lin_con;
prop     = data.props;
ther_contact = data.bc.ther_contact;

[numel,nen]   = size(icone);
[numnp,ndf]   = size(phi);
[numnp,ndm]   = size(xnod);

ijob = 'get';
handle_prop_elem;

dt       = data.props.dt;
tol      = data.props.limit;

if ndm==1,
    pgauss1d;
elseif ndm==2, 
    pgauss2d; 
elseif ndm==3,
    pgauss3d; 
end

tol      = prop.limit;
alpha    = prop.alpha;
dt       = alpha*(prop.dt);

% llevarlo a cada aplicacion
U        = alpha*phi(:,1)+(1-alpha)*phiold;
Uold     = phiold;
veloc    = data.PDE_param.veloc;
reac     = data.PDE_param.reac;
source   = data.PDE_param.source;
mass_lumped = zeros(numnp,1);
% fin de llevarlo

Rg       = forc;   % ???????????????????????????????????????????????????????????

Kele     = zeros(numel,nen,ndf,nen,ndf);
Rele     = zeros(numel,nen,ndf);

if ~isempty(ther_contact), 
    numel_th = length(ther_contact(:,1));
    Kele_th =zeros(numel_th,4,ndf,4,ndf);
    Rele_th =zeros(numel_th,4,ndf);     
end  

% call the elemset to compute the volume contribution to residual and matrix
%------------------------------------------------------------------------------
eles = (1:numel);
eval([ elemset_type '_ele;']);  % advdif_2d_ele;

% call the boundary contributions
%------------------------------------------------------------------------------
ceros = zeros(length(eles),nen,ndf);
Ue = ceros;   
for inod=1:nen,
    nodes = icone(eles,inod);    
    Ue(:,inod,1:ndf) = U(nodes,1:ndf);
end
if ~isempty(teco), 
    eval(['[Kele,Rele] = bc_teco_' elemset_type '(xnod,icone,Ue,teco,Kele,Rele,axisim);']);  % advdif_2d_ele;    
    %    [Kele,Rele] = bc_teco_advdif_2d(xnod,icone,Te,teco,Kele,Rele,ijob,axisim);
end

if weak_form,
    vele = zeros(length(eles),nen,ndm);  
    for inod=1:nen,
        nodes = icone(eles,inod);
        vele(:,inod,1:ndm) = veloc(nodes,1:ndm);       
    end    
    [Rele,Kele] = bcconv_advdif_2d(xnod,icone,vele,Ue,ifaceb,Rele,Kele,axisim);
end

% Thermal contact 
%------------------------------------------------------------------
if ~isempty(ther_contact), 
    eval(['[Kele_th,Rele_th] = bc_thermal_contact_' elemset_type '(xnod,icone,Ue,ther_contact,Kele_th,Rele_th,axisim);']);  
%    [Kele_th,Rele_th] = bc_thermal_contact(xnod,icone,T,ther_contact,Kele_th,Rele_th,ijob,axisim);
end

% Gather contributions from eles to nodes
%------------------------------------------------------------------
if ~isempty(ther_contact), 
    icone_th = ther_contact(:,1:4);
    [Rg,Kg] = gather([Rele;Rele_th],[Kele;Kele_th],[icone;icone_th],nen,ndf);
else
    [Rg,Kg] = gather(Rele,Kele,icone,nen,ndf);
end

if 0,
    Rg = Rg - diag(mass_lumped)*phi;
    Kg = Kg + diag(mass_lumped);
end

if 1,
    Rg = Rg+forc;
end

% Linear restrictions
%------------------------------------------------------------------
if isempty(lin_con)==0,[Rg,Kg] = linear_constraint(Rg,Kg,lin_con,ndf);end

% disp('fem001');keyboard;pause