% ====================================================================
%
%  Elasticity in 2D
%
% ====================================================================

ref = 2;

read_from_file = 0;
filestate = 'test_1.state';

nnwt = 1;

L        = 2;                     % Depth of the fluid
Nx       = 40;                     % Number of elements in r dir
Ny       = Nx;
xratio   = 1;               % Ratio between axis / side element size
yratio   = 1;                  % Ratio between axis / side element size
x0       = -1;
y0       = -1;

ndf      = 2;
tol      = 1e-6;
npg      = 4 ;

axisim   = 0 ;
plane_stress = 1;

% PHYSICAL DATA
rho      = 1 ;
gravity  = -9.81*0;
young    = 1e6 ;
poisson  = 0.25 ;  


% INITIAL VALUES
uini    = 1e-3*[1,1];

% ====================================================================
xnod     =[];
x        = x0+refh([1,1,1],Nx)*L;
y        = y0+refh([1,1,1],Ny)*L;
%-------- coordenadas (XNOD)
for k=1:Ny,
    xnod=[xnod;[x y(k)*ones(Nx,1)]];
end

if nen_ele==2;  % linear approximation
    type_geo_elem   = 'cartesian2d';
    %    type_geo_elem   = 'triangle';
    type_x_interp   = 'linear';
    type_sol_interp = 'linear'; 
    npg             = 2;
    icone    =[];
    ico = [ (1:Nx-1)' (2:Nx)' Nx+(2:Nx)' Nx+(1:Nx-1)'];
    for k=1:Ny-1,
        icone=[icone; (k-1)*Nx+ico ];
    end
    
    if strcmp(type_geo_elem,'triangle')==1,
        % convert quads into triangles
        quad_to_tri;
        npg             = 4;
    end      
end

[numnp,ndm] = size(xnod);
[numel,nen] = size(icone);

%-------- Solucion inicial (PHI)
if read_from_file==1,
    eval(['load ' filestate '.mat']);
    U = phi;
else
    U       =  ones(numnp,1)*uini;   
end

%-------- fijaciones (FIXA)
xmin = min(xnod);
xmax = max(xnod);
nodes_bot   = find(abs(xnod(:,2)-xmin(2))<tol);
nodes_top   = find(abs(xnod(:,2)-xmax(2))<tol);
nodes_lef   = find(abs(xnod(:,1)-xmin(1))<tol);
nodes_rig   = find(abs(xnod(:,1)-xmax(1))<tol);
nodes_wall  = rmdblist([nodes_bot;nodes_lef;nodes_rig;nodes_top]);

fixa= zeros(numnp,ndf);
fixa(nodes_top,1:ndm) = ones(length(nodes_top),ndm);
fixa(nodes_bot,1:ndm) = ones(length(nodes_bot),ndm);

%-------- Cargas/Desplazamientos nodales(FORC)
forc   = zeros(numnp,ndf);

%-------- Propiedades del material
ijob = 'set';
handle_prop_elem;

%-------- Cargas repartidas (STRG)
bou_strg = [];
if 1
    [ifaceb,nodes_bou] = pfbou2d_v3(xnod,icone,1,type_sol_interp,type_geo_elem);

    eles_rig = get_eles_by_nodes(nodes_rig,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_rig,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [ifaceb(eles_rig,1) , ones(length(eles_rig),1)*2 , -tx*[1,0]]; 
    
    eles_lef = get_eles_by_nodes(nodes_lef,nodes_bou,1);
    xc = xc_ele(xnod,nodes_bou(eles_lef,:));
    tx = young/(1+poisson)*(1-xc(:,2).^2);
    bou_strg = [bou_strg;[ifaceb(eles_lef,1) , ones(length(eles_lef),1)*4 , tx*[1,0]]]; 
end
strg = bou_strg;

%-------- Condiciones de contorno mixtas (TECO)
teco     = [];

% external volumetric force
source = zeros(numel,2);

%-------- Condiciones periodicas y deslizamiento AHORA como linear constraints
lin_con = [];
if 0,
    for icompo=[1,2,3]
        lin_con = [lin_con ; ... 
                [ones(length(nodes_lef),1) nodes_lef icompo*ones(length(nodes_lef),1) ...
                    -ones(length(nodes_rig),1) nodes_rig icompo*ones(length(nodes_rig),1)]];
        
        lin_con = [lin_con ; ... 
                [ones(length(nodes_top),1) nodes_top icompo*ones(length(nodes_top),1) ...
                    -ones(length(nodes_top),1) nodes_top-2 icompo*ones(length(nodes_top),1)]];
    end
end

% condicion inicial 
%-------------------
bini   = U;

for k=1:ndf,
    nofree         = find(fixa(:,k)~=0);
    bini(nofree,k) = forc(nofree,k);
end   

bc_time_flag = 0;
bc_time_t_end = 1;
