%
% 2D elasticity
%       ** plane stress
%       ** plane strain
%       ** axisymmetric
%

source   = data.PDE_param.source;

% Localizacion
%------------------------------------------------------------------
nele = length(eles);
ceros = zeros(nele,nen*ndf);
Ue = ceros;
U0e = ceros;
re = zeros(nele,nen);
for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    Ue(:,jnod)  = U(nodes,1:ndf);		 
    U0e(:,jnod) = Uold(nodes,1:ndf);   
    re(:,inod)  = xnod(nodes,1);		        
end

% constitutive law
lambda = poisson*young/((1+poisson)*(1-2*poisson));
mu     = young/(2*(1+poisson));

D33 = zeros(3,3);
D33(1,1) = lambda+2*mu;
D33(2,2) = lambda+2*mu;
D33(1,2) = lambda;
D33(2,1) = lambda;
D33(3,3) = mu;
D44      = lambda+2*mu;

D3 = [lambda;lambda;0];

ndf_int = 3;

if (axisim)
    D = [ [D33,D3;D3',D44]];
    ndf_int = 4;
else
    if(plane_stress)
        D33 = D33-D3*D44^(-1)*D3';
    end    
    D = D33;
end

B = zeros(nele,ndf_int,ndm);

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------

for ipg=1:npg

    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg),data.mesh.type_sol_interp,data.mesh.type_geo_elem);
    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),data.mesh.type_sol_interp,data.mesh.type_geo_elem);     
    
%    [X2S,S2X,Jaco] = xy2st(xnod,icone(eles,:),psipg(ipg),etapg(ipg));
%    [dshapex,N] = shape2d(X2S,psipg(ipg),etapg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    %    Upg    = zeros(nele,ndf);
    %    U0pg    = zeros(nele,ndf);
    radpg    = zeros(nele,1);
    epsilon = zeros(nele,ndf_int,1);
    
    for inod=1:nen,
        
        % [B] matrix
        B(:,1,1) = dshapex(:,inod,1);
        B(:,2,2) = dshapex(:,inod,2);
        B(:,3,1) = dshapex(:,inod,2);
        B(:,3,2) = dshapex(:,inod,1);        
        if axisim
            B(:,4,1) = N(inod)./radpg;
        end        
        
        jnod = (inod-1)*ndf+(1:ndf);
        
        epsilon = epsilon + rowprod(B,Ue(:,jnod),ndf_int,ndm,1);
        
        %        Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
        %        U0pg(:,(1:ndf)) = U0pg(:,(1:ndf)) + N(inod).* U0e(:,jnod);
        
        radpg(:,1)      =      radpg(:,1) + N(inod).* re(:,inod);
    end
    
    % stress
    sigma = rowprod(D,epsilon,ndf_int,ndf_int,1,-1);
    
    % Correccion axisimetrica del jacobiano de la transformacion al elemento master
    %------------------------------------------------------------------
    xsj =  wpg(ipg) .* Jaco;
    if axisim,
        xsj = xsj.*radpg;
    end
    
    for inod=1:nen,
        
        % [B] matrix
        B(:,1,1) = dshapex(:,inod,1);
        B(:,2,2) = dshapex(:,inod,2);
        B(:,3,1) = dshapex(:,inod,2);
        B(:,3,2) = dshapex(:,inod,1);        
        if axisim
            B(:,4,1) = N(inod)./radpg;
        end
        
        % B transposed
        BT = row_transp(B);
        
        % residual contributions
        ter_sigma = rowprod(BT,sigma,ndm,ndf_int,1);
        
        ter_vol_force = N(inod).*(source(eles,:));
        ter_vol_force(:,2) = ter_vol_force(:,2) + N(inod)*rho*gravity ;
        
        for idm=1:ndm
            Rele(eles,inod,idm) = Rele(eles,inod,idm) + (-ter_sigma(:,idm)+ter_vol_force(:,idm)) .* xsj;
        end
        
        % Calculo matriz tangente 
        %----------------------------
        
        for jnod=1:nen,
            
            % [B] matrix
            Bm(:,1,1) = dshapex(:,jnod,1);
            Bm(:,2,2) = dshapex(:,jnod,2);
            Bm(:,3,1) = dshapex(:,jnod,2);
            Bm(:,3,2) = dshapex(:,jnod,1);
            
            if axisim
                Bm(:,4,1) = N(jnod)./radpg;
            end                
            
            vaux = rowprod(D,Bm,ndf_int,ndf_int,ndm,-1);
            ter_M_sigma = rowprod(BT,vaux,ndm,ndf_int,ndm);
            
            for idm=1:ndm,
                for jdm=1:ndm
                    Kele(eles,inod,idm,jnod,jdm) = Kele(eles,inod,idm,jnod,jdm) + ter_M_sigma(:,idm,jdm)  .* xsj;
                end
            end
            
        end  % jnod
        
    end % inod
    
end % ipg


return
