%
% beam approximation
%

% local reference frame
ex = xnod(icone(:,2),:)-xnod(icone(:,1),:);
ex = ex./(sqrt(sum(ex'.^2))'*ones(1,3));

ey = pvec([1,0,0],ex);
eynorm = sqrt(sum(ey'.^2))';
nn = find(eynorm<1e-10);
if ~isempty(nn), 
    ey(nn,:) = pvec([0,1,0],ex(nn,:)); 
    eynorm = sqrt(sum(ey'.^2))';    
end
nn = find(eynorm<1e-10);
if ~isempty(nn), 
    ey(nn,:) = pvec([0,0,1],ex(nn,:)); 
    eynorm = sqrt(sum(ey'.^2))';    
end
nn = find(eynorm<1e-10);
if ~isempty(nn), error('***  it is impossible to orthogonalize the axial versor ***'); end

ez = pvec(ex,ey);
ez = ez./(sqrt(sum(ez'.^2))'*ones(1,3));

ey = pvec(ez,ex);
ey = ey./(sqrt(sum(ey'.^2))'*ones(1,3));

tmp = ex;
ex = ey;
ey = ez;
ez = tmp;

% element-wise nodal coordinates 
for k=1:nen
    xe(:,1:ndm,k) = xnod(icone(:,k),1:ndm); 
end

% Localizacion
%------------------------------------------------------------------
nele = length(eles);
ceros = zeros(nele,nen*ndf);
Ue_GS = ceros;
U0e_GS = ceros;
for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    Ue_GS(:,jnod)  = U(nodes,1:ndf);		 
    U0e_GS(:,jnod) = Uold(nodes,1:ndf);   
end

% transformation from global reference system to local reference system
RR(:,1:3,1) = ex;
RR(:,1:3,2) = ey;
RR(:,1:3,3) = ez;
RRT = row_transp(RR);

xe_1d = rowprod(RRT,xe,3,3,nen);
xe_1d = xe_1d(:,3,:);
xe_1d = reshape(xe_1d,numel,nen);

for inod=1:nen,    
    indx = (inod-1)*ndf+(1:3);
    Ue(:,indx) = rowprod(RRT,Ue_GS(:,indx),3,3,1);
    U0e(:,indx) = rowprod(RRT,U0e_GS(:,indx),3,3,1);    
    indx = (inod-1)*ndf+3+(1:3);
    Ue(:,indx) = rowprod(RRT,Ue_GS(:,indx),3,3,1);    
    U0e(:,indx) = rowprod(RRT,U0e_GS(:,indx),3,3,1);        
end

% icone_prop is an array with columns representing element-wise properties 
% for this element the inertia
I_xx =  icone_prop(:,1);
I_yy =  icone_prop(:,2);
I_zz =  icone_prop(:,3);
I_x  =  icone_prop(:,4);
I_y  =  icone_prop(:,5);
Area =  icone_prop(:,6);

% element applied forces per unit length
F_body = source;
F_body(:,g_dir) = (F_body(:,g_dir) + rho*gravity);
F_GS = F_body.*(Area*ones(1,ndm));
% transformation to local system
F = rowprod(RRT,F_GS,3,3,1);

% element applied couples per unit length
C_GS = [ F_body(:,3).*I_x , F_body(:,3).*I_y , (F_body(:,2).*I_y - F_body(:,1).*I_x) ];
% transformation to local system
C = rowprod(RRT,C_GS,3,3,1);

% constitutive law
lambda = poisson*young/((1+poisson)*(1-2*poisson));
mu     = young/(2*(1+poisson));

% shear stress matrix
% ver Oate pagina 122
%coef_dist = 6/5; % rectangular section
%coef_dist = 6/7; % circular section
%c=diam_int/diam_ext;K = c/(1+c^2); coef_dist = 6/(7+20*K^2); % circular hollow section
coef_dist = 1;
A_1 = coef_dist*Area;
A_2 = coef_dist*Area;
D_s = zeros(nele,2,2);
D_s(:,1,1) = mu*A_1;
D_s(:,2,2) = mu*A_2;
% bending matrix
D_b = zeros(nele,2,2);
D_b(:,1,1) = young*I_xx;
D_b(:,2,2) = young*I_yy;

ndf_int = ndf;

% axial gradient of shape functions
B_a  = zeros(nele,1,ndf_int);
B_am = zeros(nele,1,ndf_int);
B_t  = zeros(nele,1,ndf_int);
B_tm = zeros(nele,1,ndf_int);
B_b  = zeros(nele,2,ndf_int);
B_bm = zeros(nele,2,ndf_int);
B_s  = zeros(nele,2,ndf_int);
B_sm = zeros(nele,2,ndf_int);

stress = [];
% Loop sobre los puntos de Gauss
%------------------------------------------------------------------
for ipg=1:npg
    
    %    [X2S,S2X,Jaco] = xy2st(xnod_1d,icone(eles,:),psipg(ipg));
    [X2S,S2X,Jaco] = xe2se(xe_1d,psipg(ipg));    
    [dshapex,N] = shape1d(X2S,psipg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    %    Upg    = zeros(nele,ndf);
    %    U0pg    = zeros(nele,ndf);
    epsilon = zeros(nele,1);
    psi = zeros(nele,1);
    kappa = zeros(nele,2);
    gamma = zeros(nele,2);
    
    for inod=1:nen,        
        % [B_a] axial gradient matrix
        B_a(:,1,3) = dshapex(:,inod,1);        
        % [B_t] torsion gradient matrix
        B_t(:,1,6) = dshapex(:,inod,1);                
        % [B_b] bending gradient matrix
        B_b(:,1,4) = dshapex(:,inod,1);                
        B_b(:,2,5) = dshapex(:,inod,1);                
        % [B_s] shearing gradient matrix
        B_s(:,1,1) = dshapex(:,inod,1);                        
        B_s(:,2,2) = dshapex(:,inod,1);                        
        B_s(:,1,5) = -N(inod);                        
        B_s(:,2,4) =  N(inod);                        
        
        jnod = (inod-1)*ndf+(1:ndf);   
        % epsilon = B_a*d (ver Hughes pp. 375)
        epsilon = epsilon + rowprod(B_a,Ue(:,jnod),1,ndf,1);
        psi = psi + rowprod(B_t,Ue(:,jnod),1,ndf,1);
        kappa = kappa + rowprod(B_b,Ue(:,jnod),2,ndf,1);
        gamma = gamma + rowprod(B_s,Ue(:,jnod),2,ndf,1);        
    end
    
    % stress
    sigma   = young*Area.*epsilon;
    torsion = mu.*(I_xx+I_yy).*psi;    
    sigma_b = rowprod(D_b,kappa,2,2,1);
    sigma_s = rowprod(D_s,gamma,2,2,1);
    
    if compute_stress
        if 0
            sigma   = young.*epsilon;
            torsion = mu.*psi;    
            sigma_b = young*kappa;
            sigma_s = mu*gamma;
            
            stress = [stress ; [sigma,torsion,sigma_b,sigma_s]];
        else
            Upg    = zeros(nele,ndf);
            dUpg   = zeros(nele,ndf);
            
            for inod=1:nen,        
                jnod = (inod-1)*ndf+(1:ndf);
                knod = (inod-1)*ndm+(1:ndm);                
                Upg(:,(1:ndf))  =  Upg(:,(1:ndf)) + N(inod).*  Ue(:,jnod);
                dUpg(:,(1:ndf))  = dUpg(:,(1:ndf)) + (dshapex(:,inod,1)*ones(1,ndf)).* Ue(:,jnod);	 
            end

            X_max = width/2;
            Y_max = height/2;
            
            stress_13 = 2*mu*(dUpg(:,1)-Y_max.*dUpg(:,6)-Upg(:,5));
            stress_23 = 2*mu*(dUpg(:,2)+X_max.*dUpg(:,6)+Upg(:,4));
            stress_33 = young*(dUpg(:,3)-X_max.*dUpg(:,5)+Y_max.*dUpg(:,4));
            stress = [stress ; [stress_13,stress_23,stress_33]];
            % epsilon-height/2*kappa
            % keyboard;pause            
        end

        
    end
    
    % Correccion axisimetrica del jacobiano de la transformacion al elemento master
    %------------------------------------------------------------------
    xsj =  wpg(ipg) .* Jaco;
    
    for inod=1:nen,
        
        % [B_a] axial gradient matrix
        B_a(:,1,3) = dshapex(:,inod,1);        
        % [B_t] torsion gradient matrix
        B_t(:,1,6) = dshapex(:,inod,1);                
        % [B_b] bending gradient matrix
        B_b(:,1,4) = dshapex(:,inod,1);                
        B_b(:,2,5) = dshapex(:,inod,1);                
        % [B_s] shearing gradient matrix
        B_s(:,1,1) = dshapex(:,inod,1);                        
        B_s(:,2,2) = dshapex(:,inod,1);                        
        B_s(:,1,5) = -N(inod);                        
        B_s(:,2,4) =  N(inod);                        
        
        % B transposed
        B_aT = row_transp(B_a);
        B_tT = row_transp(B_t);        
        B_bT = row_transp(B_b);
        B_sT = row_transp(B_s);
        
        % residual contributions
        ter_sigma     = flag_terms(3)*rowprod(B_aT,sigma,ndf,1,1);
        ter_torsion   = flag_terms(4)*rowprod(B_tT,torsion,ndf,1,1);        
        ter_sigma_b   = flag_terms(1)*rowprod(B_bT,sigma_b,ndf,2,1);        
        ter_sigma_s   = flag_terms(2)*rowprod(B_sT,sigma_s,ndf,2,1);                
        
        ter_vol_force(:,1:3) = N(inod).*F; 
        ter_vol_force(:,3+(1:3)) = N(inod).*[C(:,1),-C(:,2),C(:,3)]; 
        
        for idf=1:ndf
            Rele(eles,inod,idf) = Rele(eles,inod,idf) + ...
                ( - ter_sigma(:,idf) - ter_torsion(:,idf) - ter_sigma_b(:,idf) - ter_sigma_s(:,idf) + ...
                ter_vol_force(:,idf) ) .* xsj;
        end
        
        % Calculo matriz tangente 
        %----------------------------
        
        for jnod=1:nen,
            
            % [B] matrix
            B_am(:,1,3) = dshapex(:,jnod,1);
            vaux = B_am; 
            vaux(:,1,3) = vaux(:,1,3).*(young*Area);
            ter_M_sigma = flag_terms(3)*rowprod(B_aT,vaux,ndf,1,ndf);
            
            B_tm(:,1,6) = dshapex(:,jnod,1);
            vaux = B_tm; 
            vaux(:,1,6) = vaux(:,1,6).*(mu*(I_xx+I_yy));
            ter_M_psi = flag_terms(4)*rowprod(B_tT,vaux,ndf,1,ndf);            
            
            B_bm(:,1,4) = dshapex(:,jnod,1);                
            B_bm(:,2,5) = dshapex(:,jnod,1);                            
            vaux = rowprod(D_b,B_bm,2,2,ndf);
            ter_M_sigma_b = flag_terms(1)*rowprod(B_bT,vaux,ndf,2,ndf);
            
            B_sm(:,1,1) = dshapex(:,jnod,1);                        
            B_sm(:,2,2) = dshapex(:,jnod,1);                        
            B_sm(:,1,5) = -N(jnod);                        
            B_sm(:,2,4) =  N(jnod);                        
            vaux = rowprod(D_s,B_sm,2,2,ndf);
            ter_M_sigma_s = flag_terms(2)*rowprod(B_sT,vaux,ndf,2,ndf);
            
            for idf=1:ndf,
                for jdf=1:ndf
                    Kele(eles,inod,idf,jnod,jdf) = Kele(eles,inod,idf,jnod,jdf) + ... 
                        ( ter_M_sigma(:,idf,jdf) + ter_M_psi(:,idf,jdf) + ter_M_sigma_b(:,idf,jdf) + ter_M_sigma_s(:,idf,jdf) )  .* xsj;
                end
            end
            
        end  % jnod
        
    end % inod
    
end % ipg

return

% Transform element-wise residual and matrix from local to global system

RRx = RR;
RRx(:,4:6,4:6) = RR;
RRxT = row_transp(RRx);

for inod=1:nen
    Rele_GS(:,inod,1:ndf) = rowprod(RRx,reshape(Rele(:,inod,:),numel,ndf),ndf,ndf,1);
    for jnod=1:nen
        vaux = rowprod(RRx,reshape(Kele(:,inod,:,jnod,:),numel,ndf,ndf),ndf,ndf,ndf);
        Kele_GS(:,inod,1:ndf,jnod,1:ndf) = rowprod(vaux,RRxT,ndf,ndf,ndf);
    end
end

Rele = Rele_GS;
Kele = Kele_GS;


