%
% beam and frame approximations
%

% local reference frame
ex = xnod(icone(:,2),:)-xnod(icone(:,1),:);
ex = ex./(sqrt(sum(ex'.^2))'*ones(1,3));

ey = pvec([1,0,0],ex);
eynorm = sqrt(sum(ey'.^2))';
nn = find(eynorm<1e-10);
if ~isempty(nn), 
    ey(nn,:) = pvec([0,1,0],ex(nn,:)); 
    eynorm = sqrt(sum(ey'.^2))';    
end
nn = find(eynorm<1e-10);
if ~isempty(nn), 
    ey(nn,:) = pvec([0,0,1],ex(nn,:)); 
    eynorm = sqrt(sum(ey'.^2))';    
end
nn = find(eynorm<1e-10);
if ~isempty(nn), error('***  it is impossible to orthogonalize the axial versor ***'); end

ez = pvec(ex,ey);
ez = ez./(sqrt(sum(ez'.^2))'*ones(1,3));

ey = pvec(ez,ex);
ey = ey./(sqrt(sum(ey'.^2))'*ones(1,3));

tmp = ex;
ex = ey;
ey = ez;
ez = tmp;

% element-wise nodal coordinates 
for k=1:nen
    xe(:,1:ndm,k) = xnod(icone(:,k),1:ndm); 
end

% Localizacion
%------------------------------------------------------------------
nele = length(eles);
ceros = zeros(nele,nen*ndf);
Ue_GS = ceros;
U0e_GS = ceros;
for inod=1:nen,
    nodes  = icone(eles,inod);
    jnod  = (inod-1)*ndf+(1:ndf);
    Ue_GS(:,jnod)  = U(nodes,1:ndf);		 
    U0e_GS(:,jnod) = Uold(nodes,1:ndf);   
end

% transformation from global reference system to local reference system
RR(:,1:3,1) = ex;
RR(:,1:3,2) = ey;
RR(:,1:3,3) = ez;
RRT = row_transp(RR);

xe_1d = rowprod(RRT,xe,3,3,nen);
xe_1d = xe_1d(:,3,:);
xe_1d = reshape(xe_1d,numel,nen);

for inod=1:nen,    
    indx = (inod-1)*ndf+(1:3);
    Ue(:,indx) = rowprod(RRT,Ue_GS(:,indx),3,3,1);
    U0e(:,indx) = rowprod(RRT,U0e_GS(:,indx),3,3,1);    
    indx = (inod-1)*ndf+3+(1:3);
    Ue(:,indx) = rowprod(RRT,Ue_GS(:,indx),3,3,1);    
    U0e(:,indx) = rowprod(RRT,U0e_GS(:,indx),3,3,1);        
end

% icone_prop is an array with columns representing element-wise properties 
% for this element the inertia
I_xx =  icone_prop(:,1);
I_yy =  icone_prop(:,2);
I_zz =  icone_prop(:,3);
I_x  =  icone_prop(:,4);
I_y  =  icone_prop(:,5);
Area =  icone_prop(:,6);

% element applied forces per unit length
F_body = source;
F_body(:,g_dir) = (F_body(:,g_dir) + rho*gravity);
F_GS = F_body.*(Area*ones(1,ndm));
% transformation to local system
F = rowprod(RRT,F_GS,3,3,1);

% element applied couples per unit length
C_GS = [ F_body(:,3).*I_x , F_body(:,3).*I_y , (F_body(:,2).*I_y - F_body(:,1).*I_x) ];
% transformation to local system
C = rowprod(RRT,C_GS,3,3,1);

% constitutive law
lambda = poisson*young/((1+poisson)*(1-2*poisson));
mu     = young/(2*(1+poisson));

% shear stress matrix
% ver Oate pagina 122
%coef_dist = 6/5; % rectangular section
%coef_dist = 6/7; % circular section
%c=diam_int/diam_ext;K = c/(1+c^2); coef_dist = 6/(7+20*K^2); % circular hollow section
coef_dist = 1;
A_1 = coef_dist*Area;
A_2 = coef_dist*Area;
D_s = diag([mu*A_1,mu*A_2]);
% bending matrix
D_b = diag([young*I_xx,young*I_yy]);

ndf_int = ndf;

% axial gradient of shape functions
B_a  = zeros(nele,1,ndf_int);
B_am = zeros(nele,1,ndf_int);

% Loop sobre los puntos de Gauss
%------------------------------------------------------------------

for ipg=1:npg
    
    %    [X2S,S2X,Jaco] = xy2st(xnod_1d,icone(eles,:),psipg(ipg));
    [X2S,S2X,Jaco] = xe2se(xe_1d,psipg(ipg));    
    [dshapex,N] = shape1d(X2S,psipg(ipg),nen); 
    
    % Interpolacion en punto de Gauss
    %------------------------------------------------------------------
    %    Upg    = zeros(nele,ndf);
    %    U0pg    = zeros(nele,ndf);
    epsilon = zeros(nele,1);
    
    for inod=1:nen,        
        % [B_a] axial gradient matrix
        B_a(:,1,3) = dshapex(:,inod,1);        
        jnod = (inod-1)*ndf+(1:ndf);   
        % epsilon = B_a*d (ver Hughes pp. 375)
        epsilon = epsilon + rowprod(B_a,Ue(:,jnod),1,ndf,1);
    end
    
    % stress
    sigma = young*Area.*epsilon;
    
    % Correccion axisimetrica del jacobiano de la transformacion al elemento master
    %------------------------------------------------------------------
    xsj =  wpg(ipg) .* Jaco;
    
    for inod=1:nen,
        
        % [B] matrix
        B_a(:,1,3) = dshapex(:,inod,1);
        
        % B transposed
        B_aT = row_transp(B_a);
        
        % residual contributions
        ter_sigma = rowprod(B_aT,sigma,ndf,1,1);
        
        ter_vol_force(:,1:3) = N(inod).*F; 
        ter_vol_force(:,3+(1:3)) = N(inod).*[C(:,1),-C(:,2),C(:,3)]; 
        
        for idf=1:ndf
            Rele(eles,inod,idf) = Rele(eles,inod,idf) + ...
                (-ter_sigma(:,idf)+ter_vol_force(:,idf)) .* xsj;
        end
        
        % Calculo matriz tangente 
        %----------------------------
        
        for jnod=1:nen,
            
            % [B] matrix
            B_am(:,1,3) = dshapex(:,jnod,1);
            vaux = B_am; 
            vaux(:,1,3) = vaux(:,1,3).*(young*Area);

            ter_M_sigma = rowprod(B_aT,vaux,ndf,1,ndf);
            
            for idf=1:ndf,
                for jdf=1:ndf
                    Kele(eles,inod,idf,jnod,jdf) = Kele(eles,inod,idf,jnod,jdf) + ... 
                        ter_M_sigma(:,idf,jdf)  .* xsj;
                end
            end
            
        end  % jnod
        
    end % inod
    
end % ipg

% Transform element-wise residual and matrix from local to global system

RRx = RR;
RRx(:,4:6,4:6) = RR;
RRxT = row_transp(RRx);

for inod=1:nen
    Rele_GS(:,inod,1:ndf) = rowprod(RRx,reshape(Rele(:,inod,:),numel,ndf),ndf,ndf,1);
    for jnod=1:nen
        vaux = rowprod(RRx,reshape(Kele(:,inod,:,jnod,:),numel,ndf,ndf),ndf,ndf,ndf);
        Kele_GS(:,inod,1:ndf,jnod,1:ndf) = rowprod(vaux,RRxT,ndf,ndf,ndf);
    end
end

Rele = Rele_GS;
Kele = Kele_GS;

return
