function [dNdx,N] = shape1d(X2S,psipg,type_interp)
%------------------------------------------------------------------
%
%       [dNdx,N] = shape1d(X2S,psipg,type_interp)
%
%-------------------------------------------------------------------

if strcmp(type_interp,'linear')
% Linear elements
psi = [-1 1 ];
% shape functions 
N(1)  = 0.5*(1+psi(1)*psipg);
N(2)  = 0.5*(1+psi(2)*psipg);
% and derivatives
dN1d1     = 0.5*psi(1);
dN2d1     = 0.5*psi(2);
dNdx(:,1) = X2S(:,1) * dN1d1 ;
dNdx(:,2) = X2S(:,1) * dN2d1 ;

elseif strcmp(type_interp,'quadratic')
% quadratic elements
N(1)   =   -psipg*(1-psipg)/2;
N(2)   =   (1-psipg)*(1+psipg);
N(3)   =   psipg*(1+psipg)/2;

dN1d1     = -0.5*(1-2*psipg);
dN2d1     = -2*psipg;
dN3d1     = 0.5*(1+2*psipg);
dNdx(:,1) = X2S(:,1) * dN1d1 ;
dNdx(:,2) = X2S(:,1) * dN2d1 ;
dNdx(:,3) = X2S(:,1) * dN3d1 ;

elseif strcmp(type_interp,'cubic')
% cubic elements
N(1) =  -9/16*(psipg+1/3)*(psipg-1/3)*(psipg-1);
N(2) = +27/16*(psipg+  1)*(psipg-1/3)*(psipg-1);
N(3) = -27/16*(psipg+  1)*(psipg+1/3)*(psipg-1);
N(4) =  +9/16*(psipg+  1)*(psipg+1/3)*(psipg-1/3);

dN1d1 =  -9/16*( ... 
  (psipg-1/3)*(psipg-1)+(psipg+1/3)*(psipg-1)+(psipg+1/3)*(psipg-1/3));
dN2d1 = +27/16*( ...
  (psipg-1/3)*(psipg-1) + (psipg+  1)*(psipg-1) + (psipg+  1)*(psipg-1/3));
dN3d1 = -27/16*( ... 
 (psipg+1/3)*(psipg-1) + (psipg+  1)*(psipg-1) + (psipg+  1)*(psipg+1/3));
dN4d1 =  +9/16*( ...
 (psipg+1/3)*(psipg-1/3)+(psipg+  1)*(psipg-1/3)+(psipg+  1)*(psipg+1/3));
	
dNdx(:,1) = X2S(:,1) * dN1d1 ;
dNdx(:,2) = X2S(:,1) * dN2d1 ;
dNdx(:,3) = X2S(:,1) * dN3d1 ;
dNdx(:,4) = X2S(:,1) * dN4d1 ;

else
  error(' ** ERROR ** , Routine SHAPE , Bad type of element')
end