function [dNdx,N] = shape1d(X2S,psipg,type_interp)
%------------------------------------------------------------------
%
%       [dNdx,N] = shape1d(X2S,psipg,type_interp)
%
%-------------------------------------------------------------------

psi = [-1 1 ];
    
if strcmp(type_interp,'linear')
    % Linear elements
    % shape functions 
    N(:,1)  = 0.5*(1+psi(1)*psipg);
    N(:,2)  = 0.5*(1+psi(2)*psipg);
    % and derivatives
    dN1d1     = 0.5*psi(1);
    dN2d1     = 0.5*psi(2);
    dNdx(:,1) = X2S(:,1) .* dN1d1 ;
    dNdx(:,2) = X2S(:,1) .* dN2d1 ;
    
elseif strcmp(type_interp,'quadratic')    
    % quadratic elements
    N(:,1)  = 0.5*(1+psi(1)*psipg);
    N(:,2)  = 0.5*(1+psi(2)*psipg);    
    N(:,3)  = 0.5*(psipg.^2-1);
    % and derivatives
    dN1d1     = 0.5*psi(1);
    dN2d1     = 0.5*psi(2);
    dN3d1     = psipg;
    
    dNdx(:,1) = X2S(:,1) .* dN1d1 ;
    dNdx(:,2) = X2S(:,1) .* dN2d1 ;
    dNdx(:,3) = X2S(:,1) .* dN3d1 ;
    
elseif strcmp(type_interp,'cubic')
    % cubic elements
    N(:,1)  = 0.5*(1+psi(1)*psipg);
    N(:,2)  = 0.5*(1+psi(2)*psipg);    
    N(:,3)  = 0.5*(psipg.^2-1);
    N(:,4)  = 1/6*(psipg.^3-psipg);
    % and derivatives
    dN1d1     = 0.5*psi(1);
    dN2d1     = 0.5*psi(2);
    dN3d1     = psipg;
    dN4d1     = 1/6*(3*psipg.^2-1);    

    dNdx(:,1) = X2S(:,1) .* dN1d1 ;
    dNdx(:,2) = X2S(:,1) .* dN2d1 ;
    dNdx(:,3) = X2S(:,1) .* dN3d1 ;
    dNdx(:,4) = X2S(:,1) .* dN4d1 ;
    
else
    error(' ** ERROR ** , Routine SHAPE , Bad type of element')
end