%
%   shape functions in natural coordinates
%
%   routine : shape2d_master
%

if strcmp(type_geometry,'cartesian2d')
    if strcmp(type_interp,'linear')
        % Linear elements
        nen = 4;
        psi = [-1 1 1 -1 ];
        eta = [-1 -1 1 1 ];
        for k=1:nen
            % shape functions 
            N(:,k)= (0.5*(1+psi(k)*psipg)).*(0.5*(1+eta(k)*etapg));
            dNdpsi(:,k,1) = (0.5*psi(k))*(0.5*(1+eta(k)*etapg));
            dNdpsi(:,k,2) = (0.5*(1+psi(k)*psipg)).*(0.5*eta(k));
        end
        
    elseif strcmp(type_interp,'quadratic')
        % Quadratic elements        
        nen = 9;
        N1d(1,1)   =   -psipg*(1-psipg)/2;
        N1d(2,1)   =   (1-psipg)*(1+psipg);
        N1d(3,1)   =   psipg*(1+psipg)/2;        
        N1d(1,2)   =   -etapg*(1-etapg)/2;
        N1d(2,2)   =   (1-etapg)*(1+etapg);
        N1d(3,2)   =   etapg*(1+etapg)/2;        
        dN1d(1,1)  =  -0.5*(1-2*psipg);
        dN1d(2,1)  =  -2*psipg;
        dN1d(3,1)  =   0.5*(1+2*psipg);
        dN1d(1,2)  =  -0.5*(1-2*etapg);
        dN1d(2,2)  =  -2*etapg;
        dN1d(3,2)  =   0.5*(1+2*etapg);
        
        for k1=1:sqrt(nen)
            for k2=1:sqrt(nen)            
                k= (k1-1)*sqrt(nen)+k2;
                N(:,k) =  N1d(k2,1).*N1d(k1,2);
                dNdpsi(:,k,1) = dN1d(k2,1).*N1d(k1,2);
                dNdpsi(:,k,2) = N1d(k2,1).*dN1d(k1,2);
            end            
        end
    end
    
elseif strcmp(type_geometry,'triangle')
    if strcmp(type_interp,'linear')
        % Linear elements
        nen = 3;
        if 1
            % N = a+b*psi+c*eta
            % coef = [a,b,c]
            coef = [1,-1,-1 ; 0,1,0 ; 0,0,1];
            for k=1:nen
                N(:,k) = coef(k,1)+coef(k,2)*psipg+coef(k,3)*etapg;
                dNdpsi(:,k,1) = coef(k,2);
                dNdpsi(:,k,2) = coef(k,3);
            end
        else
            N(:,1)  = 1-psipg-etapg;
            N(:,2)  = psipg;
            N(:,3)  = etapg;        
            k=1;dNdpsi(:,k,1:2) = [-1,-1]; 
            k=2;dNdpsi(:,k,1:2) = [+1, 0];         
            k=3;dNdpsi(:,k,1:2) = [ 0,+1];         
        end
    elseif strcmp(type_interp,'quadratic')
        % Quadratic elements        
        nen = 6;
        coef = [ ...
                1     0     0     0     0     0
               -3    -1     0     4     0     0
               -3     0    -1     0     0     4
                2     2     0    -4     0     0
                4     0     0    -4     4    -4
                2     0     2     0     0    -4  ];
        coef = coef';        
        for k=1:nen
            N(:,k) = coef(k,1)+coef(k,2)*psipg+coef(k,3)*etapg+coef(k,4)*psipg.^2+coef(k,5)*psipg.*etapg+coef(k,6)*etapg.^2;
            dNdpsi(:,k,1) = coef(k,2)+2*coef(k,4).*psipg+coef(k,5).*etapg;
            dNdpsi(:,k,2) = coef(k,3)+2*coef(k,6).*etapg+coef(k,5).*psipg;;
        end
    else
        error(' ** ERROR ** , Routine SHAPE2D_MASTER , Bad type of interpolation')        
    end    
else
    error(' ** ERROR ** , Routine SHAPE2D_MASTER , Bad type of element')    
end
