function [phi,Rg]=solver(phi,Rg,Kg,fixa,prop,ndf)
%
%    [phi,Rg]=solver(phi,Rg,Kg,fixa,prop,ndf)
%
%
%   SOLVER routine for linear algebra 
%   
%   Type_solver = 0 ---> direct method 
%               = 1 ---> Line search
%               = 2 ---> Explicit - Forward Euler
%               = 3 ---> GMRES   
%               = 4 ---> QMR (quasi minimum residual) 
%

type_solver = prop.type_solver;
relax       = prop.relax;
nu_max      = prop.nu_max;
alfa_min    = prop.alfa_min;
alpha       = prop.alpha;
ntime       = prop.ntime;
nstep       = prop.nstep;
scaling     = prop.scaling;

[numnp,ndf] = size(phi);

if scaling & type_solver~=2,
    % SCALING
    scalev = sum(abs(Kg'))';
    Rg   = Rg ./ scalev;
    Kg   = diag(scalev)\Kg;
    % END OF SCALING
end

if type_solver==0,
    
    free = find(fixa==0);
    Rg = Rg(free);
    Kg = Kg(free,free);
    
    dphi = Kg\Rg;
    phiv = m2v(phi);
    phiv(free) = phiv(free)+ relax*dphi;
    dphi = relax*dphi;
    phi  = v2m(phiv,ndf);
    
elseif type_solver==1,
    
    linesear
    
elseif type_solver==2,
    
    free = find(fixa==0);
    Rg = Rg(free);
    Kg = diag(Kg);
    Kg = Kg(free);     
    dphi = Rg./Kg;
    phiv = m2v(phi);
    phiv(free) = phiv(free)+ dphi;
    phi  = v2m(phiv,ndf);
    
elseif type_solver==3,
    
    free = find(fixa==0);
    Rg = Rg(free);
    Kg = Kg(free,free);
    
    TOL = 1e-6;
    MAXIT = 25;
    RESTART = 25;

    if 0,
        M1=[]; M2=[]; 
    elseif 1,
        M1 = diag(diag(Kg));
        M2 = speye(size(Kg,1));
    else
        [M1,M2] = luinc(Kg,TOL); 
    end
    phiv = m2v(phi);     
    X0 = phiv(free);
    
    disp(' ===== SOLVER ROUTINE =====');    
    
    t=cputime;
    [dphi,FLAG,RELRES,ITER,RESVEC] = GMRES(Kg,Rg,RESTART,TOL,MAXIT,M1,M2,X0);
    t_el = cputime-t;
    disp([' GMRES method --- CPU time = '  num2str(t_el) ])
    
    disp([' GMRES --- flag & |R| ' num2str(FLAG) '  &  ' num2str(RELRES) ])
    
    phiv(free) = phiv(free)+ dphi;
    phi  = v2m(phiv,ndf);
    
elseif type_solver==4,
    
    free = find(fixa==0);
    Rg = Rg(free);
    Kg = Kg(free,free);
    
    TOL = 1e-6;
    MAXIT = 25;
    RESTART = 25;
    %keyboard;pause
    if 0,
        M1=[]; M2=[]; 
    elseif 1,
        M1 = diag(diag(Kg));
        M2 = speye(size(Kg,1));
    else
        [M1,M2] = luinc(Kg,TOL); 
    end
    phiv = m2v(phi);     
    X0 = phiv(free);
    
    disp(' ===== SOLVER ROUTINE =====');    
    t=cputime;
    dphi = qmr(Kg,Rg,TOL,500,[],[],[]);
    t_el = cputime-t;
    disp([' QMR method --- CPU time = '  num2str(t_el) ])
    
    phiv(free) = phiv(free)+ dphi;
    phi  = v2m(phiv,ndf);
        
else
    error(' Bad choice of Solver type [0,1,2] ')
end
